library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity rcv is
  port (
    rst        : in  std_logic;
    clk_x16    : in  std_logic;
    data_out   : out std_logic_vector (7 downto 0);
    RxD        : in  std_logic;
    data_ready : out std_logic;
    strobe     : in  std_logic
    );
end rcv;

architecture Behavioral of rcv is
  signal rcv_done, strobe_last, clk_x1, clk_x2, ready, ser_in,
         clk_x2_last: std_logic;
  signal counter : std_logic_vector (3 downto 0);
  signal shift_register : std_logic_vector (7 downto 0);
  signal bitcount, num : std_logic_vector (3 downto 0);
begin

  divider : process (clk_x16, rst)
  begin
    if (rst = '1') then
      counter <= "0000";
    elsif (rising_edge(clk_x16)) then
      if (rcv_done = '1') then
        counter <= "1100";
      else
        counter <= counter + 1;
      end if;
    end if;
  end process divider;
  clk_x1 <= counter(3);
  clk_x2 <= counter(2);

  sampler : process (clk_x16, rst)
  begin
    if (rst = '1') then
      num     <= "1111";
    elsif (rising_edge(clk_x16)) then
      if (RxD = '1' and num /= "1111") then
        num <= num + 1;
      elsif (RxD = '0' and num /= "0000") then
        num <= num - 1;
      else
        num <= num;
      end if;
    end if;
  end process sampler;
  ser_in <= '1' when (num >= "1000") else '0';
  
 done : process (clk_x16, rst)
  begin
    if (rst='1') then
      rcv_done <= '1';
    elsif (rising_edge(clk_x16)) then
      -- Start bit received, start receiving
      if (ser_in='0') then
        rcv_done <= '0';
      elsif (bitcount = "1001") then
        rcv_done <= '1';
      end if;
    end if;
  end process done;    

  clear : process (clk_x16, rst)
  begin 
    if (rst='1') then
      ready <= '1';
      strobe_last <= '0';
    elsif (rising_edge(clk_x16)) then
      strobe_last <= strobe;
      if (ser_in='0') then
        ready <= '0';
      elsif (strobe = '1' and strobe_last <= '0') then
        ready <= '1';
      end if;
    end if;
  end process clear;    

  shift : process (clk_x16, rst)
  begin
    if (rst='1') then
      bitcount <= "0000";
      shift_register <= "00000000";
      clk_x2_last <= '0';
    elsif (rising_edge(clk_x16)) then
        clk_x2_last <= clk_x2;
      if (clk_x2 = '1' and clk_x2_last = '0' and
          clk_x1 = '0' and rcv_done = '0') then
        bitcount <= bitcount + 1;
        shift_register <= ser_in & shift_register(7 downto 1);
      elsif (rcv_done = '1') then
        bitcount <= "0000";
      end if;
    end if;
  end process shift;

  data_out <= shift_register;
  data_ready <= '1' when (rcv_done = '1' and ready='0') else '0';

end Behavioral;
