library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity tx is
  port (
    rst        : in  std_logic;
    clk_x16    : in  std_logic;
    data_in    : in std_logic_vector (7 downto 0);
    TxD        : out  std_logic;
    data_sent  : out std_logic;
    strobe     : in  std_logic
    );
end tx;

architecture Behavioral of tx is
  signal tx_done, strobe_last, clk_x1: std_logic;
  signal counter : std_logic_vector (3 downto 0);
  signal shift_register : std_logic_vector (8 downto 0);
  signal bitcount : std_logic_vector (3 downto 0);
--  signal tdr : std_logic_vector (7 downto 0) ;
begin
  divider : process (clk_x16, rst)
  begin
    if (rst = '1') then
      counter <= "0000";
    elsif (rising_edge(clk_x16)) then
      if (tx_done = '1' and strobe = '0') then
        counter <= "0000";
      else
        counter <= counter - 1;
      end if;
    end if;
  end process divider;
  clk_x1 <= counter(3);

  done_send : process (clk_x16, rst)
  begin
    if (rst='1') then
      tx_done <= '1';
    elsif (rising_edge(clk_x16)) then
      if (strobe = '1') then
        tx_done <= '0';
      elsif (bitcount = "1010") then
        tx_done <= '1';
      end if;
    end if;
  end process done_send;    

  shift : process (clk_x1, rst)
  begin
    if (rst='1') then
      bitcount <= "0000";
      shift_register <= "111111111";
      strobe_last <= '0';
    elsif (rising_edge(clk_x1)) then
      strobe_last <= strobe;
      if (strobe = '1' and strobe_last = '0') then
        bitcount <= "0000";
        shift_register <= data_in & '0';
      elsif (tx_done = '0') then
        bitcount <= bitcount + 1;
        -- Shift right
        shift_register <= '1' & shift_register(8 downto 1);
      end if; 
    end if;
  end process shift;

  data_sent <= '1' when (tx_done = '1') else '0';
  TxD <= (shift_register(0));

end Behavioral;
